import java.awt.*;
import java.awt.event.*;

public class HeadItem extends Object
{
  String itemTitle;
  int itemIndex;

  Dimension size, dragArea;
  private Component Parent;
  private Font parentFont;
  private boolean mouseIn, pressed, inDragArea;
  private int x, y;

  // Class initialization code
  HeadItem(Component parent, Font font, String title) {
    Parent = parent;
    itemIndex = 0;
    itemTitle = new String(title);
    parentFont = font;
    inDragArea = false;
    FontMetrics fm = Parent.getFontMetrics(font);
    int height  = fm.getHeight();
    height += 8;
    int width = 12 + fm.stringWidth(itemTitle);
    size = new Dimension(width, height);
    dragArea = new Dimension(4, height);
  }

  public int getItemIndex() { return itemIndex; };

  public void setItemIndex(int index) { itemIndex = index; };

  public String getItemTitle() { return itemTitle; };

  public void setItemTitle(String title) { itemTitle = title; };

  public Dimension getSize() { 
    return size;
  };

  public void setSize(Dimension sz) {
    size = new Dimension(sz);
  };

  public void setWidth(int width) { 
    size.width = width;
  };

  public boolean isPressed() { return pressed; }

  public boolean isMouseIn() { return mouseIn; }

  public boolean isInDragArea() { return inDragArea; }

  public void setOrigin(int xx, int yy)
  {
    x = xx;
    y = yy;
  }

  public boolean mouseEnter(MouseEvent event)
  {
    mouseIn = true;
    if ((event.getX() > this.x + size.width - dragArea.width && 
        event.getX() < this.x + size.width) && 
        (event.getY() > this.y && event.getY() < this.y + size.height))
      inDragArea = true;
    else
      inDragArea = false;
    repaint();
    return true;
  }

  // Mouse button pressed handling code
  public boolean mouseExit(MouseEvent event)
  {
    mouseIn = false;
    inDragArea = false;
    repaint();
    return true;
  }

  // Mouse button pressed handling code
  public boolean mouseDown(MouseEvent event)
  {
    // Right click sets META_MASK
    if ((event.getModifiers() & MouseEvent.META_MASK) == 0)
      if (mouseIn) {
        pressed = true;
        if ((event.getX() > this.x + size.width - dragArea.width && 
            event.getX() < this.x + size.width) && 
           (event.getY() > this.y && event.getY() < this.y + size.height))
          inDragArea = true;
        else
          inDragArea = false;
      }
    repaint();
    return true;
  }

  // Mouse button released handling code
  public boolean mouseUp(MouseEvent event)
  {
    // Right click sets META_MASK
    if ((event.getModifiers() & MouseEvent.META_MASK) == 0)
      if (pressed && mouseIn) {
         pressed = false;
         repaint();
      }
    return true;
  }

  public boolean hasPoint(int x, int y)
  {
    if (x > this.x && x < this.x + size.width)
      if (y > this.y && y < this.y + size.height)
        return true;
    return false;
  }

  public void repaint()
  {
    paint(Parent.getGraphics());
  }

  // This routine handles the painting
  public void paint(Graphics graphic)
  {
    FontMetrics fm = Parent.getFontMetrics(parentFont);
    graphic.setFont(parentFont);
    graphic.setColor(Color.lightGray);
    graphic.fillRect(x, y, size.width, size.height);
    graphic.setColor(Color.black);
    graphic.fillRect(x + size.width - dragArea.width, y, dragArea.width, size.height);
    String text = itemTitle;
    int endindex = text.length();
    while ((endindex > 0) &&
           (fm.stringWidth(text.substring(0, endindex)) > size.width - 6))
      endindex -= 1;
    String outString = new String(text.substring(0, endindex));
    if (pressed && !inDragArea)
    {
      graphic.setColor(Color.darkGray);
      graphic.drawLine( x, y, x + size.width, y);
      graphic.drawLine( x, y, x, y + size.height);
      graphic.drawString(outString, x + 5, y + size.height - 4);
    }
    else
    {
      graphic.setColor(Color.white);
      graphic.drawLine( x, y, x + size.width, y);
      graphic.drawLine( x, y, x, y + size.height);
      graphic.drawLine( x + 1, y + 1, x + size.width - 1, y + 1);
      graphic.drawLine( x + 1, y + 1, x + 1, y + size.height - 1);
      graphic.setColor(Color.black);
      graphic.drawLine( x + 1, y + size.height - 1, x + size.width, y + size.height - 1);
      graphic.drawLine( x + size.width, y + 1, x + size.width, y + size.height);
      graphic.drawLine( x + 2, y + size.height - 2, x + size.width - 1, y + size.height - 2);
      graphic.drawLine( x + size.width - 1, y + 2, x + size.width - 1, y + size.height - 1);
      graphic.drawString(outString, x + 4, y + size.height - 5);
    } 
  }
} // end class HeadItem
