import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import HeadItem;

public class Header extends Component implements MouseListener,
                      MouseMotionListener, ComponentListener
{
  Vector headerItems;
  Vector pushListeners = new Vector();
  int headerCount;

  private Font itemFont;
  private Cursor oldCursor;
  private HeadItem selected;
  private int selItemIndex;
  private int modifiers;
  private int changedWitdh, xx;
  private boolean dragMode;
   
  // Class initialization code
  Header(int hCount) {
    itemFont = new Font("Serif", Font.PLAIN, 14);
    headerCount = hCount;
    headerItems = new Vector();
    selItemIndex = 0;
    dragMode = false;
    addComponentListener(this);
    addMouseListener(this);
    addMouseMotionListener(this);
  }

  public HeadItem addHeadItem(HeadItem item) {
    headerItems.addElement(item);
    return item;
  }

  public HeadItem addHeadItem(String title) {
    HeadItem item = new HeadItem(this, itemFont, title);
    headerItems.addElement(item);
    return item;
  }

  public void removeHeadItem(HeadItem item) {
    headerItems.removeElement(item);
  }

  public void removeAll() {
    headerItems.removeAllElements();
    repaint();
  }

  public Dimension getPreferredSize() {
    if (headerItems.size() > 0) {
      int width = 0;
      int height = 0;
      for (int i = 0; i < headerItems.size(); i++)
      {
        HeadItem t = (HeadItem) headerItems.elementAt(i);
        if (t != null) {
          width += t.getSize().width;
          height = t.getSize().height;
        }
      }
      return new Dimension(width, height);
    }
    else 
      return new Dimension(100, 50);
  }
  
  public Dimension getMinimumSize() {
    return new Dimension(100, 50);
  }

  public synchronized void addActionListener(ActionListener l) {
    pushListeners.addElement(l);
  }
 
  public synchronized void removeActionListener(ActionListener l) {
    pushListeners.removeElement(l);
  }
 
  public synchronized void reSize() {
    if (headerItems.size() > 0) {
      int len = 0;
      int dx = getSize().width / headerItems.size();
      int ddx = getSize().width % headerItems.size();
      for (int i = 0; i < headerItems.size(); i++)
      {
        HeadItem t = (HeadItem) headerItems.elementAt(i);
        t.setItemIndex(i);
        if (i < headerItems.size() - 1)
          t.setWidth(dx);
        else
          t.setWidth(dx + ddx);
      }
    }
    repaint();
  }

  public synchronized void reArrainge() {
    if (headerItems.size() > 0) {
      int ofs = 0;
      for (int i = 0; i < headerItems.size(); i++)
      {
        HeadItem t = (HeadItem) headerItems.elementAt(i);
        t.setOrigin(ofs, 0);
        ofs += t.getSize().width;
      }
    }
  }

  public void performAction() {
    if (selected != null) {
      selItemIndex = selected.getItemIndex();
      Vector targets;
      synchronized (this) {
        targets = (Vector) pushListeners.clone();
      }
      HeadItem t = (HeadItem) headerItems.elementAt(selItemIndex);
      ActionEvent actionEvt = new ActionEvent(this,
                         ActionEvent.ACTION_PERFORMED, 
                         Integer.toString(t.getItemIndex()), modifiers);
      for (int i = 0; i < targets.size(); i++) {
        ActionListener target = (ActionListener) targets.elementAt(i);
        target.actionPerformed(actionEvt);
      }
    }
  }

  public void componentHidden(ComponentEvent e) {}
  public void componentMoved(ComponentEvent e) {} 
  public void componentResized(ComponentEvent e) { reSize(); reArrainge();
    System.out.println("Header Resized !");
  } 
  public void componentShown(ComponentEvent e) {}

  public void mouseClicked(MouseEvent e) {}

  public void mousePressed(MouseEvent e) {
    if (headerItems.size() > 0) {
      for (int i = 0; i < headerItems.size(); i++)
      {
        HeadItem t = (HeadItem) headerItems.elementAt(i);
        if (t.hasPoint(e.getX(), e.getY())) {
          t.mouseDown(e);
          if (t.isInDragArea()) {
            dragMode = true;
            oldCursor = getCursor();
//            setCursor(oldCursor.getPredefinedCursor(Cursor.W_RESIZE_CURSOR));
            xx = e.getX();
            changedWitdh = t.getSize().width;
          }
        }
      }
    }
  }

  public void mouseReleased(MouseEvent e) {
    if (headerItems.size() > 0) {
      boolean doAction = false;
      if (dragMode && selected != null) {
        modifiers = ActionEvent.ACTION_LAST + 1;
        changedWitdh += e.getX() - xx;
        selected.setWidth(changedWitdh);
        selected.mouseUp(e);
//        setCursor(oldCursor);
        doAction = true;
        dragMode = false;
        reArrainge();
        repaint();
      } else {
        for (int i = 0; i < headerItems.size(); i++)
        {
          HeadItem t = (HeadItem) headerItems.elementAt(i);
          if (t.hasPoint(e.getX(), e.getY())) {
            if ((e.getModifiers() & MouseEvent.META_MASK) == 0) {
              modifiers = ActionEvent .ACTION_LAST + 2;
              doAction = true;
            }
          }
          t.mouseUp(e);
        }
      }
      if (doAction)
        performAction();
    }
    requestFocus();
  }

  public void mouseEntered(MouseEvent e) {
    if (headerItems.size() > 0) {
      for (int i = 0; i < headerItems.size(); i++)
      {
        HeadItem t = (HeadItem) headerItems.elementAt(i);
        if (t.hasPoint(e.getX(), e.getY())) {
          selected = t;
          t.mouseEnter(e);
          selItemIndex = selected.getItemIndex();
        }
      }
    }
  }

  public void mouseExited(MouseEvent e) {
    if (headerItems.size() > 0) {
      if (selected != null) {
        selected.mouseExit(e);
        selected = null;
      }
    }
  }

  public void mouseDragged(MouseEvent e) {
    if (dragMode && selected != null) {
      changedWitdh += e.getX() - xx;
      xx = e.getX();
      selected.setWidth(changedWitdh);
      reArrainge();
      repaint();
    }
  }

  public void mouseMoved(MouseEvent e) {
    if (headerItems.size() > 0) {
      for (int i = 0; i < headerItems.size(); i++)
      {
        HeadItem t = (HeadItem) headerItems.elementAt(i);
        if (t.hasPoint(e.getX(), e.getY())) {
          if (selected != t) {
//            setCursor(oldCursor);
            dragMode = false;
            if (selected != null)
              selected.mouseExit(e);
            selected = t;
            t.mouseEnter(e);
            selItemIndex = selected.getItemIndex();
          }
        }
      }
    }
  }

  // This routine handles the painting
  public synchronized void paint(Graphics graphic)
  {
    System.out.println("Header Repainted !");
    graphic.setFont(itemFont);
    graphic.setColor(Color.gray);
//    graphic.setColor(getBackground());
    graphic.fillRect( 0, 0, getSize().width, getSize().height);
    if (headerItems != null && headerItems.size() > 0) {
      for (int i = 0; i < headerItems.size(); i++)
      {
        ((HeadItem) headerItems.elementAt(i)).paint(graphic);
      }
    }
  }

} // end class Header
