import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import ListItemDetail;

public class ListItem extends Object
{
  Image itemIcon;
  int itemIndex;
  String itemTitle;
  Vector itemDetails;
  int detailCount;
  int viewMode;

  boolean pressed;
  Dimension size, sizeDetailed;
  Dimension[] drawSize;
  private Component Parent;
  private Font parentFont;
  private boolean mouseIn;
  private int x, y;

  // Class initialization code
  ListItem(Component parent, Font font, String iconFile, String title, int dcount) {
    this(parent, font, iconFile, title, dcount, ListView.ICONVIEW);
  }

  ListItem(Component parent, Font font, String iconFile, String title,
           int dcount, int mode) {
    Parent = parent;
    itemIcon = Parent.getToolkit().getImage(iconFile);
    itemIndex = 0;
    itemTitle = new String(title);
    viewMode = mode;
    itemDetails = new Vector();
    detailCount = dcount;
    drawSize = new Dimension[dcount];
    parentFont = font;
    FontMetrics fm = Parent.getFontMetrics(font);
    int height  = (itemIcon.getHeight(Parent) > fm.getHeight()) ?
                     itemIcon.getHeight(Parent) : fm.getHeight();
    height += 4;
    int width = 8 + itemIcon.getWidth(Parent) + fm.stringWidth(itemTitle);
    size = new Dimension(width, height);
    sizeDetailed = new Dimension(4 + itemIcon.getWidth(Parent), height);
  }

  public int getItemIndex() { return itemIndex; };

  public void setItemIndex(int index) { itemIndex = index; };

  public Image getItemIcon() { return itemIcon; };

  public void setItemIcon(Image icon) { itemIcon = icon; };

  public String getItemTitle() { return itemTitle; };

  public void setItemTitle(String title) { itemTitle = title; };

  public int getDetailCount() { return detailCount; };

  public void setDetailCount(int count) { detailCount = count; };

  public Dimension getSize() { 
    if (viewMode == ListView.ICONVIEW)
      return size;
    else
      return sizeDetailed;
  };

  public void setSize(Dimension sz) {
    if (viewMode == ListView.ICONVIEW)
      size = new Dimension(sz);
    else
      sizeDetailed = new Dimension(sz);
  };

  public Dimension getDrawSize(int index) { return drawSize[index]; }

  public void setDrawSize(int index, Dimension sz) {
    drawSize[index] = new Dimension(sz);
  }

  public void setDrawWidth(int index, int newWidth) {
    int nwidth = sizeDetailed.width + newWidth - drawSize[index].width;
    sizeDetailed = new Dimension(nwidth, sizeDetailed.height);
    drawSize[index].width = newWidth;
  }

  public int getWidth(int length) {
    FontMetrics fm = Parent.getFontMetrics(parentFont);
    return (8 + itemIcon.getWidth(Parent) + 
            fm.getMaxAdvance() * length);
  }

  public void setWidth(int width) { 
    if (viewMode == ListView.ICONVIEW)
      size.width = width;
    else
      sizeDetailed.width = width;
  };

  public int getViewMode() { return viewMode; };

  public void setViewMode(int mode) { 
    if (viewMode != mode)
      viewMode = mode;
  };

  public void addItemDetail(String text) {
    addItemDetail(new ListItemDetail(0, text));
  }

  public void addItemDetail(ListItemDetail detail) {
    FontMetrics fm = Parent.getFontMetrics(parentFont);
    detail.setDetailIndex(itemDetails.size());
    drawSize[itemDetails.size()] = new Dimension();
    drawSize[itemDetails.size()].width = fm.stringWidth(detail.detailText);
    itemDetails.addElement(detail);
    sizeDetailed.width += 4 + fm.stringWidth(detail.detailText);
  }

  public void removeItemDetail(ListItemDetail detail) {
    itemDetails.removeElement(detail);
//    detailCount = itemDetails.size();
  }

  public boolean isPressed() { return pressed; }

  public boolean isMouseIn() { return mouseIn; }

  public void setOrigin(int xx, int yy)
  {
    x = xx;
    y = yy;
  }

  public boolean mouseEnter(MouseEvent event)
  {
    mouseIn = true;
    repaint();
    return true;
  }

  // Mouse button pressed handling code
  public boolean mouseExit(MouseEvent event)
  {
    mouseIn = false;
//    pressed = false;
    repaint();
    return true;
  }

  // Mouse button pressed handling code
  public boolean mouseDown(MouseEvent event)
  {
    // Right click sets META_MASK
    if ((event.getModifiers() & MouseEvent.META_MASK) == 0)
      if (mouseIn)
        pressed = true;
    repaint();
    return true;
  }

  // Mouse button released handling code
  public boolean mouseUp(MouseEvent event)
  {
    // Right click sets META_MASK
    if ((event.getModifiers() & MouseEvent.META_MASK) == 0)
      if (pressed && mouseIn) {
         pressed = false;
         repaint();
      }
    return true;
  }

  public boolean hasPoint(int x, int y)
  {
    Dimension sz;
    if (viewMode == ListView.ICONVIEW)
      sz = size;
    else
      sz = sizeDetailed;
    if (x > this.x && x < this.x + sz.width)
      if (y > this.y && y < this.y + sz.height)
        return true;
    return false;
  }

  public void repaint()
  {
    paint(Parent.getGraphics());
  }

  // This routine handles the painting
  public void paint(Graphics graphic)
  {
    FontMetrics fm = Parent.getFontMetrics(parentFont);
    graphic.setFont(parentFont);
    if (itemIcon != null) 
    {
      Dimension sz;
      graphic.drawImage(itemIcon, x + 2, y + 2, itemIcon.getWidth(Parent),
                        itemIcon.getHeight(Parent), Parent);
      graphic.setColor(Color.black);
      if (viewMode == ListView.ICONVIEW) {
        graphic.drawString(itemTitle, x + itemIcon.getWidth(Parent) + 4,
                           y + itemIcon.getHeight(Parent) - 4);
        sz = size;
      }
      else {
        int xx = x + itemIcon.getWidth(Parent) + 4;
        int yy = y + itemIcon.getHeight(Parent) - 4;
        for (int i = 0; i < itemDetails.size(); i++) {
          ListItemDetail d = (ListItemDetail) itemDetails.elementAt(i);
          String text = d.getDetailText();
          int endindex = text.length();
          while ((endindex > 0) &&
          (fm.stringWidth(text.substring(0, endindex)) > drawSize[i].width))
            endindex -= 1;
          String outString = new String(text.substring(0, endindex));
          graphic.drawString(outString, xx, yy);
          xx += 4 + drawSize[i].width;
        }
        sz = sizeDetailed;
      }
      if (mouseIn)
      {
        graphic.setColor(Color.black);
        graphic.drawRect(x, y, sz.width, sz.height);
      }
      else
      {
        graphic.setColor(Parent.getBackground());
        graphic.drawRect(x, y, sz.width, sz.height);
      } 
   }
  }
} // end class ListItem
