import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import ListItemDetail;
import ListItem;
import ListView;
import HeadItem;

public class ListPanel extends Component implements MouseListener,
                      MouseMotionListener, KeyListener,
                      ComponentListener, Adjustable, ItemSelectable
{
  Vector listItems;
  int viewMode;
  HeadItem[] columnHeaders;
  int detailCount;
  Vector pushListeners = new Vector();

  private int dx, dy;
  private ListItem selected;
  private Font itemFont;
  private int frameWidth;
  private int frameHeight;
  private int areaWidth;
  private int areaHeight;
  private int selItemIndex;
  private int firstColumn, firstRow;
  private int selColumn, selRow;
//  private int firstIndex, endIndex;

  public static final int ICONVIEW = 0;
  public static final int DETAILVIEW = 1;
   
  // Class initialization code
  ListPanel(int dcount) {
    this(dcount, ListView.ICONVIEW);
  }

  ListPanel(int dcount, int mode) {
    itemFont = new Font("Serif", Font.PLAIN, 14);
    detailCount = dcount;
    viewMode = mode;
    listItems = new Vector();
    columnHeaders = new HeadItem[dcount];
    selItemIndex = 0;
    firstColumn = 0;
    selColumn = 0;
    firstRow = 0;
    selRow = 0;
    frameWidth = 0;
    areaWidth = 0;
    frameHeight = 0;
    areaHeight = 0;
    addComponentListener(this);
    addKeyListener(this);
    addMouseListener(this);
    addMouseMotionListener(this);
  }

//  public int getSelectedColumn() { return selColumn; };

  public int getViewMode() { return viewMode; };

  public void setViewMode(int mode) { 
    if (viewMode != mode) {
      viewMode = mode;
      resetView(viewMode);
    }
  };

  public void resetView(int mode) {
    for (int i = 0; i < listItems.size(); i++)
      ((ListItem) listItems.elementAt(i)).setViewMode(mode);
//    if (selected != null)
//      selected.setViewMode(mode);
    reSize();
    reArrainge();
    repaint();
  }

  public void addHeadItem(HeadItem header, int index) {
    columnHeaders[index] = header;
  }

  public void setHeadSize(int index) {
    int newWidth = columnHeaders[index].getSize().width;
    if (listItems.size() > 0) {
      for (int i = 0; i < listItems.size(); i++)
      {
        ListItem t = (ListItem) listItems.elementAt(i);
        t.setDrawWidth(index, newWidth);
      }
    }
    repaint();
  }

  public ListItem addListItem(ListItem item) {
    listItems.addElement(item);
    return item;
  }

  public ListItem addListItem(String iconFile, String title) {
    ListItem item = new ListItem(this, itemFont, iconFile, title, detailCount);
    listItems.addElement(item);
    return item;
  }

  public void removeListItem(ListItem item) {
    listItems.removeElement(item);
  }

  public void removeAll() {
    listItems.removeAllElements();
    selItemIndex = 0;
    firstColumn = 0;
    selColumn = 0;
    firstRow = 0;
    selRow = 0;
    frameWidth = 0;
    areaWidth = 0;
    frameHeight = 0;
    areaHeight = 0;
    selected = null;
    repaint();
  }

  public Dimension getPreferredSize() {
    return new Dimension(0, 0);
  }
  
  public Dimension getMinimumSize() {
    return new Dimension(0, 0);
  }

  public synchronized void addActionListener(ActionListener l) {
    pushListeners.addElement(l);
  }
 
  public synchronized void removeActionListener(ActionListener l) {
    pushListeners.removeElement(l);
  }
 
  public synchronized void reSize() {
    if (listItems.size() > 0) {
      int len = 0;
      dx = 0; dy = 0;
      for (int i = 0; i < listItems.size(); i++)
      {
        ListItem t = (ListItem) listItems.elementAt(i);
        t.setItemIndex(i);
        dx = Math.max(t.getSize().width, dx);
        dy = Math.max(t.getSize().height, dy);
      }
      for (int i = 0; i < listItems.size(); i++)
        ((ListItem) listItems.elementAt(i)).setWidth(dx);
      if (viewMode == ListView.ICONVIEW) {
        areaHeight = (int) getSize().height / dy;
        areaWidth = (listItems.size() % areaHeight == 0) ?
                                   (int) listItems.size() / areaHeight :
                                   (int) listItems.size() / areaHeight + 1;
        frameWidth = (getSize().width % dx == 0) ? 
                                   (int) getSize().width / dx :
                                   (int) getSize().width / dx + 1;
        selColumn = selItemIndex / dx;
        selRow = selItemIndex % dx;
      }
      else { // viewMode == ListView.DETAILVIEW
        frameHeight = (int) getSize().height / dy;
        frameWidth = 1;
        areaHeight = listItems.size();
        areaWidth = 1;
        selColumn = 0;
        selRow = selItemIndex;
      }
    }
  }

  public int getUnitSize() {
    if (viewMode == ListView.ICONVIEW)
      return areaWidth;
    else
      return areaHeight;
  }

  public synchronized void reArrainge() {
    if (listItems.size() > 0) {
      int r = 0, c = 0;
      int endIndex;
      if (viewMode == ListView.ICONVIEW) {
        endIndex =
           (((firstColumn + frameWidth) * areaHeight) < listItems.size()) ? 
              ((firstColumn + frameWidth) * areaHeight) : listItems.size();
        for (int i = firstColumn * areaHeight; i < endIndex; i++)
        {
          ListItem t = (ListItem) listItems.elementAt(i);
          t.setOrigin(c * dx, r * dy);
          r += 1;
          if (r >= areaHeight) {
            r = 0; c += 1;
          }
        }
      }
      else {
        endIndex = ((firstRow + frameHeight) < listItems.size()) ?
                      (firstRow + frameHeight) : listItems.size();
        for (int i = firstRow; i < endIndex; i++, r++)
        {
          ListItem t = (ListItem) listItems.elementAt(i);
          t.setOrigin(c, r * dy);
        }
      }
    }
  }

  public void moveUp() {
    if (selected != null) {
      selItemIndex = selected.getItemIndex();
      selected.mouseExit(null);
    }
    if (viewMode == ListView.ICONVIEW) {
      if (selItemIndex > 0) {
        selItemIndex -= 1;
        selColumn = selItemIndex / areaHeight;
        selRow = selItemIndex % areaHeight;
        if (selColumn < firstColumn) {
          if (firstColumn > 0)
            firstColumn -= 1;
          reArrainge();
          repaint();
        }
      }
    }
    else
    {
      if (selItemIndex > 0) {
        selItemIndex -= 1;
        selRow = selItemIndex;
        if (selRow < firstRow) {
          if (firstRow > 0)
            firstRow -= 1;
          reArrainge();
          repaint();
        }
      }
    }
    selected = (ListItem) listItems.elementAt(selItemIndex);
    selected.mouseEnter(null);
  }

  public void moveDown() {
    if (selected != null) {
      selItemIndex = selected.getItemIndex();
      selected.mouseExit(null);
    }
    if (viewMode == ListView.ICONVIEW) {
      if (selItemIndex < listItems.size() - 1) {
        selItemIndex += 1;
        selColumn = selItemIndex / areaHeight;
        selRow = selItemIndex % areaHeight;
        if (selColumn >= firstColumn + frameWidth) {
          if (firstColumn + frameWidth + 1 <= areaWidth)
            firstColumn += 1;
          reArrainge();
          repaint();
        }
      }
    }
    else
    {
      if (selItemIndex < listItems.size() - 1) {
        selItemIndex += 1;
        selRow = selItemIndex;
        if (selRow >= firstRow + frameHeight) {
          if (firstRow + frameHeight + 1 <= areaHeight)
            firstRow += 1;
          reArrainge();
          repaint();
        }
      }
    }
    selected = (ListItem) listItems.elementAt(selItemIndex);
    selected.mouseEnter(null);
  }

  public void moveLeft() {
    if (selected != null) {
      selItemIndex = selected.getItemIndex();
      selected.mouseExit(null);
    }
    if (viewMode == ListView.ICONVIEW) {
      if (selItemIndex >= areaHeight) {
        selItemIndex -= areaHeight;
        selColumn = selItemIndex / areaHeight;
        selRow = selItemIndex % areaHeight;
        if (selColumn < firstColumn) {
          if (firstColumn > 0)
            firstColumn -= 1;
          reArrainge();
          repaint();
        }
      }
    }
    else
    {
    }
    selected = (ListItem) listItems.elementAt(selItemIndex);
    selected.mouseEnter(null);
  }

  public void moveRight() {
    if (selected != null) {
      selItemIndex = selected.getItemIndex();
      selected.mouseExit(null);
    }
    if (viewMode == ListView.ICONVIEW) {
      if ((selItemIndex < listItems.size() - areaHeight) ||
          (firstColumn + frameWidth + 1 <= areaWidth)) {
        selItemIndex = (selItemIndex < listItems.size() - areaHeight) ?
                        selItemIndex + areaHeight : listItems.size() - 1;
        selColumn = selItemIndex / areaHeight;
        selRow = selItemIndex % areaHeight;
        if (selColumn >= firstColumn + frameWidth) {
          if (firstColumn + frameWidth + 1 <= areaWidth)
            firstColumn += 1;
          reArrainge();
          repaint();
        }
      }
    }
    else
    {
    }
    selected = (ListItem) listItems.elementAt(selItemIndex);
    selected.mouseEnter(null);
  }

  public void performAction() {
    if (selected != null) {
      selItemIndex = selected.getItemIndex();
      Vector targets;
      synchronized (this) {
        targets = (Vector) pushListeners.clone();
      }
      ListItem t = (ListItem) listItems.elementAt(selItemIndex);
      ActionEvent actionEvt = new ActionEvent(this,
                         ActionEvent.ACTION_PERFORMED, t.getItemTitle());
      for (int i = 0; i < targets.size(); i++) {
        ActionListener target = (ActionListener) targets.elementAt(i);
        target.actionPerformed(actionEvt);
      }
    }
  }

  public void performAdjust() {
    Vector targets;
    adjustValue = (viewMode == ListView.ICONVIEW) ? selColumn : selRow;
    synchronized (this) {
      targets = (Vector) adjustListeners.clone();
    }
    AdjustmentEvent aEvt = new AdjustmentEvent(this,
                                  AdjustmentEvent.ADJUSTMENT_VALUE_CHANGED,
                                       AdjustmentEvent.TRACK, adjustValue);
    for (int i = 0; i < targets.size(); i++) {
      AdjustmentListener target = (AdjustmentListener)targets.elementAt(i);
      target.adjustmentValueChanged(aEvt);
    }
  }

  public void moveKeySelection(int keyCode) {
    int oldSelColumn = selColumn;
    switch(keyCode) {
      case KeyEvent.VK_UP:
        moveUp();
        break;
      case KeyEvent.VK_DOWN:
        moveDown();
        break;
      case KeyEvent.VK_LEFT:
        moveLeft();
        break;
      case KeyEvent.VK_RIGHT:
        moveRight();
        break;
      case KeyEvent.VK_ENTER:
        performAction();
        break;
//        case KeyEvent.VK_PAGE_UP:
//        case KeyEvent.VK_PAGE_DOWN:
    }
    if (oldSelColumn != selColumn)
      performAdjust();
    requestFocus();
  }

  public void componentHidden(ComponentEvent e) {}
  public void componentMoved(ComponentEvent e) {} 
  public void componentResized(ComponentEvent e) { reSize(); reArrainge(); 
    System.out.println("ListPanel Resized !");
  } 
  public void componentShown(ComponentEvent e) {}

  public void keyPressed(KeyEvent e) {} 
  public void keyReleased(KeyEvent e) { moveKeySelection(e.getKeyCode()); } 
  public void keyTyped(KeyEvent e) {}

  public void mouseClicked(MouseEvent e) {}

  public void mousePressed(MouseEvent e) {
    if (listItems.size() > 0) {
      if (selected != null) selected.mouseDown(e);
/*
      int firstIndex, endIndex;
      if (viewMode == ListView.ICONVIEW) {
        firstIndex = firstColumn * areaHeight;
        endIndex =
          (((firstColumn + frameWidth) * areaHeight) < listItems.size()) ? 
             ((firstColumn + frameWidth) * areaHeight) : listItems.size();
      }
      else
      {
        firstIndex = firstRow;
        endIndex = ((firstRow + frameHeight) < listItems.size()) ?
                    (firstRow + frameHeight) : listItems.size();
      }
      for (int i = firstIndex; i < endIndex; i++)
      {
        ListItem t = (ListItem) listItems.elementAt(i);
        t.pressed = false;
        if (t.hasPoint(e.getX(), e.getY()))
          t.mouseDown(e);
      }
*/
    }
  }

  public void mouseReleased(MouseEvent e) {
    if (listItems.size() > 0) {
      if (selected != null) {
        selected.mouseUp(e);
        performAction();
      }
/*
      int firstIndex, endIndex;
      if (viewMode == ListView.ICONVIEW) {
        firstIndex = firstColumn * areaHeight;
        endIndex =
          (((firstColumn + frameWidth) * areaHeight) < listItems.size()) ? 
             ((firstColumn + frameWidth) * areaHeight) : listItems.size();
      }
      else
      {
        firstIndex = firstRow;
        endIndex = ((firstRow + frameHeight) < listItems.size()) ?
                    (firstRow + frameHeight) : listItems.size();
      }
      for (int i = firstIndex; i < endIndex; i++)
      {
        ListItem t = (ListItem) listItems.elementAt(i);
        if (t.hasPoint(e.getX(), e.getY())) {
          if ((e.getModifiers() & MouseEvent.META_MASK) == 0)
            if (t.isPressed() && t.isMouseIn())
              doAction = true;
          t.mouseUp(e);
        }
      }
*/
      requestFocus();
    }
  }

  public void mouseEntered(MouseEvent e) {
    if (listItems.size() > 0) {
/*
      if (selected == null) {
        selItemIndex = 0;
        selected = (ListItem) listItems.elementAt(selItemIndex);
        selected.mouseEnter(e);
      }
*/
//      if (selected != null) selected.mouseEnter(e);
/*
    int firstIndex, endIndex;
    if (viewMode == ListView.ICONVIEW) {
      firstIndex = firstColumn * areaHeight;
      endIndex =
          (((firstColumn + frameWidth) * areaHeight) < listItems.size()) ? 
             ((firstColumn + frameWidth) * areaHeight) : listItems.size();
    }
    else
    {
      firstIndex = firstRow;
      endIndex = ((firstRow + frameHeight) < listItems.size()) ?
                    (firstRow + frameHeight) : listItems.size();
    }
    for (int i = firstIndex; i < endIndex; i++)
    {
      ListItem t = (ListItem) listItems.elementAt(i);
      if (t.hasPoint(e.getX(), e.getY())) {
        selected = t;
        t.mouseEnter(e);
      }
    }
*/
    }
  }

  public void mouseExited(MouseEvent e) {
    if (listItems.size() > 0) {
//      if (selected != null) selected.mouseExit(e);
/*
      int firstIndex, endIndex;
      if (viewMode == ListView.ICONVIEW) {
        firstIndex = firstColumn * areaHeight;
        endIndex =
          (((firstColumn + frameWidth) * areaHeight) < listItems.size()) ? 
             ((firstColumn + frameWidth) * areaHeight) : listItems.size();
      }
      else
      {
        firstIndex = firstRow;
        endIndex = ((firstRow + frameHeight) < listItems.size()) ?
                    (firstRow + frameHeight) : listItems.size();
      }
      for (int i = firstIndex; i < endIndex; i++)
      {
        ListItem t = (ListItem) listItems.elementAt(i);
        t.mouseExit(e);
      }
      if (selected != null)
      {
        selected.pressed = false;
        selected = null;
      }
*/
    }
  }

  public void mouseDragged(MouseEvent e) {}

  public void mouseMoved(MouseEvent e) {
    if (listItems.size() > 0) {
      int oldSel = (viewMode == ListView.ICONVIEW) ? selColumn : selRow;
      int firstIndex, endIndex;
      if (viewMode == ListView.ICONVIEW) {
        firstIndex = firstColumn * areaHeight;
        endIndex =
           (((firstColumn + frameWidth) * areaHeight) < listItems.size()) ? 
              ((firstColumn + frameWidth) * areaHeight) : listItems.size();
      }
      else
      {
        firstIndex = firstRow;
        endIndex = ((firstRow + frameHeight) < listItems.size()) ?
                      (firstRow + frameHeight) : listItems.size();
      }
      for (int i = firstIndex; i < endIndex; i++)
      {
        ListItem t = (ListItem) listItems.elementAt(i);
//            t.mouseExit(e);
        if (t.hasPoint(e.getX(), e.getY())) {
          if (selected != t) {
            if (selected != null)
              selected.mouseExit(e);
            changeSelection(selected, ItemEvent.DESELECTED);
            changeSelection(t, ItemEvent.SELECTED);
            selected = t;
            t.mouseEnter(e);
          }
        }
      }
      if (selected != null) {
        selItemIndex = selected.getItemIndex();
        if (viewMode == ListView.ICONVIEW) {
          selColumn = selItemIndex / areaHeight;
          selRow = selItemIndex % areaHeight;
          if (oldSel != selColumn)
            performAdjust();
        } else {
          selColumn = 0;
          selRow = selItemIndex;
          if (oldSel != selRow)
            performAdjust();
        }
      }
    }
  }

  // This routine handles the painting
  public synchronized void paint(Graphics graphic)
  {
    System.out.println("ListPanel Repainted !");
    graphic.setFont(itemFont);
    graphic.setColor(getBackground());
    graphic.fillRect( 0, 0, getSize().width, getSize().height);
    if (listItems != null && listItems.size() > 0) {
      int firstIndex, endIndex;
      if (viewMode == ListView.ICONVIEW) {
        firstIndex = firstColumn * areaHeight;
        endIndex =
          (((firstColumn + frameWidth) * areaHeight) < listItems.size()) ? 
             ((firstColumn + frameWidth) * areaHeight) : listItems.size();
      }
      else
      {
        firstIndex = firstRow;
        endIndex = ((firstRow + frameHeight) < listItems.size()) ?
                      (firstRow + frameHeight) : listItems.size();
      }
      for (int i = firstIndex; i < endIndex; i++)
      {
        ((ListItem) listItems.elementAt(i)).paint(graphic);
      }
    }
  }

/* Adjustable implementation */
  private Vector itemListeners = new Vector();

  public Object[] getSelectedObjects() {
    Object[] sels = new Object[1];
    sels[0] = (Object) listItems.elementAt(selItemIndex);
    return sels;
  }

  public synchronized void addItemListener(ItemListener l) {
    itemListeners.addElement(l);
  }

  public synchronized void removeItemListener(ItemListener l) {
    itemListeners.removeElement(l);
  }

  public void changeSelection(ListItem item, int stateChange) {
    if (item != null) {
      Vector targets;
      synchronized (this) {
        targets = (Vector) itemListeners.clone();
      }
      ItemEvent iEvt = new ItemEvent(this, ItemEvent.ITEM_STATE_CHANGED,
                                    (Object) item, stateChange);
      for (int i = 0; i < targets.size(); i++) {
        ItemListener target = (ItemListener) targets.elementAt(i);
        target.itemStateChanged(iEvt);
      }
    }
  }

/* Adjustable implementation */
  private Vector adjustListeners = new Vector();
  private int adjustType;
  private int adjustValue;
  private int visibleAmount;

  public synchronized void addAdjustmentListener(AdjustmentListener l) {
    adjustListeners.addElement(l);
  }
 
  public synchronized void removeAdjustmentListener(AdjustmentListener l) {
    adjustListeners.removeElement(l);
  }
 
  public int getOrientation() { return Adjustable.HORIZONTAL; }

  public void setMinimum(int min) {}

  public int getMinimum() { return 0; }

  public void setMaximum(int max) {}

  public int getMaximum() { return areaWidth; }

  public void setUnitIncrement(int u) {}

  public int getUnitIncrement() { return 1; }

  public void setBlockIncrement(int b) {}

  public int getBlockIncrement() { return 1; }

  public void setVisibleAmount(int v) { visibleAmount = v; }

  public int getVisibleAmount() { return visibleAmount; }

  public void setValue(int v) { adjustValue = v; }

  public int getValue() { return adjustValue; }
} // end class ListPanel
