import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import ListItem;
import ListPanel;
import HeadItem;
import Header;

public class ListView extends Panel implements ComponentListener, 
                         ActionListener, FocusListener, AdjustmentListener
{
  int detailCount;
  int viewMode;
  Header header;

  private Scrollbar scroller;
  private int scrolHeight;
  private int scrolWidth;
  private ListPanel listPanel;

  public static final int ICONVIEW = 0;
  public static final int DETAILVIEW = 1;
   
  // Class initialization code
  ListView(int dcount) {
    this(dcount, ListView.ICONVIEW);
  }

  ListView(int dcount, int mode) {
    detailCount = dcount;
    viewMode = mode;
    addComponentListener(this);
    addFocusListener(this);
    header = new Header(dcount);
    header.addActionListener(this);
    if (viewMode == ListView.ICONVIEW)
      scroller = new Scrollbar(Scrollbar.HORIZONTAL);
    else
      scroller = new Scrollbar(Scrollbar.VERTICAL);
    scroller.addAdjustmentListener(this);
    scrolHeight = 16;
    scrolWidth = 16;
    listPanel = new ListPanel(dcount, mode);
    listPanel.addAdjustmentListener(this);
    add(listPanel);
    add(scroller);
//    setLayout(new BorderLayout());
    setLayout(null);
  }

  public void Clear() {
    listPanel.removeAll();
  }

  public void resetView() {
    listPanel.resetView(viewMode);
    reSize();
  }

  public int getViewMode() { return viewMode; };

  public void setViewMode(int mode) { 
    if (viewMode != mode) {
      viewMode = mode;
      listPanel.setViewMode(viewMode);
      reSize();
    }
  };

  public HeadItem addColumnHeader(String title, int headerIndex) {
    HeadItem newHeader = header.addHeadItem(title);
    listPanel.addHeadItem(newHeader, headerIndex);
    return newHeader;
  }

  public void resetHeadSize() {
    for (int i = 0; i < detailCount; i++)
      listPanel.setHeadSize(i);
  }

  public ListItem addListItem(ListItem item) {
    return listPanel.addListItem(item);
  }

  public ListItem addListItem(String iconFile, String title) {
    return listPanel.addListItem(iconFile, title);
  }

  public void removeListItem(ListItem item) {
    listPanel.removeListItem(item);
  }

  public Dimension getPreferredSize() {
    return new Dimension(100, 50);
  }
  
  public Dimension getMinimumSize() {
    return new Dimension(100, 50);
  }

  public synchronized void addActionListener(ActionListener l) {
    listPanel.addActionListener(l);
  }
 
  public synchronized void removeActionListener(ActionListener l) {
    listPanel.removeActionListener(l);
  }

  public synchronized void addItemListener(ItemListener l) {
    listPanel.addItemListener(l);
  }

  public synchronized void removeItemListener(ItemListener l) {
    listPanel.removeItemListener(l);
  }

  public void componentHidden(ComponentEvent e) {}
  public void componentMoved(ComponentEvent e) {} 
  public void componentResized(ComponentEvent e) { reSize(); resetHeadSize();
    System.out.println("ListView Resized !");
  } 
  public void componentShown(ComponentEvent e) {}

  public synchronized void reSize() {
    if (viewMode == ListView.ICONVIEW) {
      scroller.setOrientation(Scrollbar.HORIZONTAL);
      remove(header);
/*
      header.setVisible(false);
      remove(scroller);
      remove(listPanel);
      add(scroller, "South");
      add(listPanel, "Center");
*/
      listPanel.setBounds(0, 0, getSize().width,
                          getSize().height - scrolHeight);
      scroller.setBounds(0, getSize().height - scrolHeight + 1, 
                         getSize().width, scrolHeight);
    }
    else {
      scroller.setOrientation(Scrollbar.VERTICAL);
      add(header);
/*
      header.setVisible(true);
      remove(header);
      remove(scroller);
      remove(listPanel);
      add(header, "North");
      add(scroller, "East");
      add(listPanel, "Center");
*/
      int hdrHeight = header.getPreferredSize().height;
      header.setBounds(0, 0, getSize().width, hdrHeight);
      listPanel.setBounds(0, hdrHeight + 1, getSize().width - scrolWidth,
                          getSize().height - hdrHeight);
      scroller.setBounds(getSize().width - scrolWidth + 1, hdrHeight + 1, 
                         scrolWidth, getSize().height - hdrHeight);
    }
    scroller.setMaximum(listPanel.getUnitSize());
    scroller.setMinimum(0);
    scroller.setUnitIncrement(1);
    scroller.setBlockIncrement(1);
    resetHeadSize();
    listPanel.requestFocus();
//    doLayout();
  }

  public void actionPerformed(ActionEvent e) {
    Integer index = Integer.valueOf(e.getActionCommand());
    if (e.getModifiers() == ActionEvent.ACTION_LAST + 1) {
      // Drag Action
      listPanel.setHeadSize(index.intValue());
    }
    else if (e.getModifiers() == ActionEvent.ACTION_LAST + 2) {
      // Click Action
    }
  }

  public void adjustmentValueChanged(AdjustmentEvent e) {
    changeAdjustment(e);
  }

  public void changeAdjustment(AdjustmentEvent e) {
    if (viewMode == ListView.ICONVIEW) {
      switch(e.getAdjustmentType()) {
        case AdjustmentEvent.UNIT_INCREMENT:
          listPanel.moveRight();
          break;
        case AdjustmentEvent.UNIT_DECREMENT:
          listPanel.moveLeft();
          break;
        case AdjustmentEvent.BLOCK_INCREMENT:
          listPanel.moveRight();
          break;
        case AdjustmentEvent.BLOCK_DECREMENT:
          listPanel.moveLeft();
          break;
        case AdjustmentEvent.TRACK:
          scroller.setValue(listPanel.getValue());
          break;
      }
    }
    else {
      switch(e.getAdjustmentType()) {
        case AdjustmentEvent.UNIT_INCREMENT:
          listPanel.moveDown();
          break;
        case AdjustmentEvent.UNIT_DECREMENT:
          listPanel.moveUp();
          break;
        case AdjustmentEvent.BLOCK_INCREMENT:
          listPanel.moveDown();
          break;
        case AdjustmentEvent.BLOCK_DECREMENT:
          listPanel.moveUp();
          break;
        case AdjustmentEvent.TRACK:
          scroller.setValue(listPanel.getValue());
          break;
      }
    }
    listPanel.requestFocus();
  }

  public synchronized void paint(Graphics graphic)
  {
    System.out.println("ListView Repainted !");
  }

  public void focusGained(FocusEvent e) {
    listPanel.requestFocus();
  }

  public void focusLost(FocusEvent e) {}

} // end class ListView
