import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import ToolButton;
import ToolSeparator;

//public class ToolBar extends Component
public class ToolBar extends Component implements MouseListener,
				 MouseMotionListener {
  Tool selected;
  Vector toolbuttons;
  int orientation;
  Point origin;
  Vector pushListeners = new Vector();

  public static final int HORIZONTAL = 0;
  public static final int VERTICAL = 1;
   
// Class initialization code
  public ToolBar()
  {
    selected = null;
    orientation = ToolBar.HORIZONTAL;
    toolbuttons = new Vector(1);
    ResetOrigin();
    addMouseListener(this);
    addMouseMotionListener(this);
  }

  public void ResetOrigin()
  {
    origin = new Point(0, 0);
  }

  public Dimension getPreferredSize() {
    if (toolbuttons.size() > 0)
      if (orientation == ToolBar.HORIZONTAL) {
        return new Dimension(origin.x, ((Tool) toolbuttons.elementAt(0)).height());
      }
      else if (orientation == ToolBar.VERTICAL) {
        return new Dimension(((Tool) toolbuttons.elementAt(0)).width(), origin.y);
      }
    return new Dimension(100, 50);
  }
  
  public Dimension getMinimumSize() {
      return new Dimension(100, 50);
  }

   public synchronized void addActionListener(ActionListener l) {
	pushListeners.addElement(l);
   }
 
   public synchronized void removeActionListener(ActionListener l) {
	pushListeners.removeElement(l);
   }
 
    public void mouseClicked(MouseEvent e) {
    }

    public void mousePressed(MouseEvent e) {
       for (int i = 0; i < toolbuttons.size(); i++)
       {
          Tool t = (Tool) toolbuttons.elementAt(i);
          t.pressed = false;
          if (t.hasPoint(e.getX(), e.getY()))
             t.mouseDown(e);
       }
    }

    public void mouseReleased(MouseEvent e) {
        for (int i = 0; i < toolbuttons.size(); i++)
        {
            Tool t = (Tool) toolbuttons.elementAt(i);
            if (t.hasPoint(e.getX(), e.getY())) 
                t.mouseUp(e);
        }
/*
        if (selected != null)
        {
            selected.mouseExit(e);
            selected.pressed = false;
            selected = null;
        }
        else
          selected = t;
*/
    }

    public void mouseEntered(MouseEvent e) {
        for (int i = 0; i < toolbuttons.size(); i++)
        {
          Tool t = (Tool) toolbuttons.elementAt(i);
          if (t.hasPoint(e.getX(), e.getY())) {
            selected = t;
            t.mouseEnter(e);
          }
        }
    }

    public void mouseExited(MouseEvent e) {
        for (int i = 0; i < toolbuttons.size(); i++)
        {
          Tool t = (Tool) toolbuttons.elementAt(i);
          t.mouseExit(e);
        }
        if (selected != null)
        {
          selected.pressed = false;
          selected = null;
        }
    }

    public void mouseDragged(MouseEvent e) {
    }

    public void mouseMoved(MouseEvent e) {
        for (int i = 0; i < toolbuttons.size(); i++)
        {
          Tool t = (Tool) toolbuttons.elementAt(i);
//          t.mouseExit(e);
          if (t.hasPoint(e.getX(), e.getY())) {
            if (selected != t) {
              if (selected != null)
                selected.mouseExit(e);
              selected = t;
              t.mouseEnter(e);
            }
          }
        }
    }

  // This routine handles the painting
  public synchronized void paint(Graphics graphic)
  {
    graphic.setColor(getBackground());
    graphic.fillRect( 0, 0, getSize().width, getSize().height);
    if (toolbuttons != null) 
      for (int i = 0; i < toolbuttons.size(); i++)
      {
        ((Tool) toolbuttons.elementAt(i)).paint(graphic);
      }
  }

  public void addButton(MenuItem mI, String bmpfile)
  {
    Tool t = new ToolButton(this, bmpfile, mI, origin, orientation);
    toolbuttons.addElement(t);
    if (orientation == ToolBar.HORIZONTAL) {
      origin.x += t.width();
      setSize(origin.x, (origin.y > t.height()) ? origin.y : t.height());
    }
    else if (orientation == ToolBar.VERTICAL) {
      origin.y += t.height();
      setSize((origin.x > t.width()) ? origin.x : t.width(), origin.y);
    }
  }

  public void addSeparator()
  {
    ToolSeparator t = new ToolSeparator(this, origin, orientation);
    toolbuttons.addElement(t);
    if (orientation == ToolBar.HORIZONTAL) {
      origin.x += t.width();
      setSize(origin.x, (origin.y > t.height()) ? origin.y : t.height());
    }
    else if (orientation == ToolBar.VERTICAL) {
      origin.y += t.height();
      setSize((origin.x > t.width()) ? origin.x : t.width(), origin.y);
    }
  }

  public int getOrientation()
  {
    return orientation;
  }

  public void setOrientation(int orient)
  {
    if (orientation != orient) {
      orientation = orient;
      ResetOrigin();
      for (int i = 0; i < toolbuttons.size(); i++) {
        Tool t = (Tool) toolbuttons.elementAt(i);
        t.setOrigin(origin, orientation);
        if (orientation == ToolBar.HORIZONTAL) {
          origin.x += t.width();
          setSize(origin.x, (origin.y > t.height()) ? origin.y : t.height());
        }
        else if (orientation == ToolBar.VERTICAL) {
          origin.y += t.height();
          setSize((origin.x > t.width()) ? origin.x : t.width(), origin.y);
        }
      }
    }
  }
} // end class ToolBar
