import java.awt.*;
import java.awt.event.*;
import java.util.Vector;
import Tool;

public class ToolButton extends Tool
{
  Image bitmap;
  MenuItem menuEq;
  Component Parent;

  // Class initialization code
  ToolButton(Component parent, String bmpfile, MenuItem m, Point origin, int orient)
  {
    Parent = parent;
    bitmap = Parent.getToolkit().getImage(bmpfile);
    pressed = false;
    mouseIn = false;
    menuEq = m;
    ImgWidth = 20;
    ImgHeight = 20;
    x = origin.x;
    y = origin.y;
  }

  public int width()
  {
    return ImgWidth + 4;
  }

  public int height()
  {
    return ImgHeight + 4;
  }

  public void setOrigin(Point origin, int orient)
  {
    x = origin.x;
    y = origin.y;
  }

  public boolean mouseEnter(MouseEvent event)
  {
    mouseIn = true;
    repaint();
    return true;
  }

  // Mouse button pressed handling code
  public boolean mouseExit(MouseEvent event)
  {
    mouseIn = false;
//    pressed = false;
    repaint();
    return true;
  }

  // Mouse button pressed handling code
  public boolean mouseDown(MouseEvent event)
  {
    // Right click sets META_MASK
    if ((event.getModifiers() & MouseEvent.META_MASK) == 0)
      if (mouseIn)
        pressed = true;
    repaint();
    return true;
  }

  // Mouse button released handling code
  public boolean mouseUp(MouseEvent event)
  {
    // Right click sets META_MASK
    if ((event.getModifiers() & MouseEvent.META_MASK) == 0)
      if (pressed && mouseIn) {
         Vector targets;
         synchronized (this) {
            targets = (Vector) ((ToolBar)Parent).pushListeners.clone();
         }
	   ActionEvent actionEvt = new ActionEvent(menuEq,
                         ActionEvent.ACTION_PERFORMED, menuEq.getLabel());
         for (int i = 0; i < targets.size(); i++) {
            ActionListener target = (ActionListener)targets.elementAt(i);
            target.actionPerformed(actionEvt);
         }
         pressed = false;
         repaint();
      }
    return true;
  }

  public boolean hasPoint(int x, int y)
  {
    if (x > this.x && x < this.x + ImgWidth + 4)
      if (y > this.y && y < this.y + ImgHeight + 4)
        return true;
    return false;
  }

  public void repaint()
  {
    paint(Parent.getGraphics());
  }

  // This routine handles the painting
  public void paint(Graphics graphic)
  {
    if (bitmap != null) 
    {
      if (pressed && mouseIn)
      {
        graphic.drawImage(bitmap, x + 2, y + 2, ImgWidth, ImgHeight, Parent);
        graphic.setColor(Color.black);
        graphic.drawLine( x, y, x + ImgWidth + 3, y);
        graphic.drawLine( x, y, x, y + ImgHeight + 3);
        graphic.drawLine( x + 1, y + 1, x + ImgWidth + 2, y + 1);
        graphic.drawLine( x + 1, y + 1, x + 1, y + ImgHeight + 2);
        graphic.setColor(Color.white);
        graphic.drawLine( x + 2, y + ImgHeight + 2, x + ImgWidth + 2, y + ImgHeight + 2);
        graphic.drawLine( x + ImgWidth + 2, y + 2, x + ImgWidth + 2, y + ImgHeight + 2);
        graphic.drawLine( x + 1, y + ImgHeight + 3, x + ImgWidth + 3, y + ImgHeight + 3);
        graphic.drawLine( x + ImgWidth + 3, y + 1, x + ImgWidth + 3, y + ImgHeight + 3);
      }
      else //if (mouseIn)
      {
        graphic.drawImage(bitmap, x + 2, y + 2, ImgWidth, ImgHeight, Parent);
        graphic.setColor(Color.white);
        graphic.drawLine( x, y, x + ImgWidth + 3, y);
        graphic.drawLine( x, y, x, y + ImgHeight + 3);
        graphic.drawLine( x + 1, y + 1, x + ImgWidth + 2, y + 1);
        graphic.drawLine( x + 1, y + 1, x + 1, y + ImgHeight + 2);
        graphic.setColor(Color.black);
        graphic.drawLine( x + 2, y + ImgHeight + 2, x + ImgWidth + 2, y + ImgHeight + 2);
        graphic.drawLine( x + ImgWidth + 2, y + 2, x + ImgWidth + 2, y + ImgHeight + 2);
        graphic.drawLine( x + 1, y + ImgHeight + 3, x + ImgWidth + 3, y + ImgHeight + 3);
        graphic.drawLine( x + ImgWidth + 3, y + 1, x + ImgWidth + 3, y + ImgHeight + 3);
      }
/*      else
      {
        graphic.setColor(Parent.getBackground());
        graphic.drawLine( x, y, x + ImgWidth + 4, y);
        graphic.drawLine( x, y, x, y + ImgHeight + 4);
        graphic.drawLine( x + 1, y + 1, x + ImgWidth + 3, y + 1);
        graphic.drawLine( x + 1, y + 1, x + 1, y + ImgHeight + 3);
        graphic.drawLine( x + 1, y + ImgHeight + 3, x + ImgWidth + 2, y + ImgHeight + 3);
        graphic.drawLine( x + ImgWidth + 3, y + 2, x + ImgWidth + 3, y + ImgHeight + 2);
        graphic.drawLine( x + 2, y + ImgHeight + 4, x + ImgWidth + 1, y + ImgHeight + 4);
        graphic.drawLine( x + ImgWidth + 4, y + 1, x + ImgWidth + 4, y + ImgHeight + 4);
        graphic.drawImage(bitmap, x + 2, y + 2, ImgWidth, ImgHeight, Parent);
      } */
   }
  }
} // end class ToolButton

